/*
 *
 *  Copyright (C) 1997-2010, OFFIS e.V.
 *  All rights reserved.  See COPYRIGHT file for details.
 *
 *  This software and supporting documentation were developed by
 *
 *    OFFIS e.V.
 *    R&D Division Health
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *
 *  Module:  dcmjp2k
 *
 *  Author:  Marco Eichelberg
 *
 *  Purpose: singleton class that registers the decoder for all supported JPEG 2000 processes.
 *
 *  Last Update:      $Author$
 *  Update Date:      $Date$
 *  CVS/RCS Revision: $Revision$
 *  Status:           $State$
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef D2DECODE_H
#define D2DECODE_H

#include "dcmtk/config/osconfig.h"
#include "dcmtk/ofstd/oftypes.h"  /* for OFBool */
#include "dcmtk/dcmjp2k/d2utils.h"

class D2CodecParameter;
class D2JPEG2000Decoder;

/** singleton class that registers the decoder for all supported JPEG 2000 processes.
 */
class D2DecoderRegistration 
{
public: 

  /** registers decoder for all supported JPEG 2000 processes.
   *  If already registered, call is ignored unless cleanup() has
   *  been performed before.
   *  @param uidcreation flag indicating whether or not
   *    a new SOP Instance UID should be assigned upon decompression.
   *  @param planarconfig flag indicating how planar configuration
   *    of color images should be encoded upon decompression.
   */   
  static void registerCodecs(
    J2K_UIDCreation uidcreation = EJ2UC_default,
    J2K_PlanarConfiguration planarconfig = EJ2PC_restore);
    
  /** deregisters decoders.
   *  Attention: Must not be called while other threads might still use
   *  the registered codecs, e.g. because they are currently decoding
   *  DICOM data sets through dcmdata.
   */  
  static void cleanup();

private:

  /// flag indicating whether the decoders are already registered.
  static OFBool registered_;

  /// pointer to codec parameter shared by all decoders
  static D2CodecParameter *cp_;
  
  /// pointer to decoder 
  static D2JPEG2000Decoder *decoder_;
  
};

#endif

/*
 * CVS/RCS Log
 * $Log$
 * Revision 1.4  2010-10-14 09:28:07  joergr
 * Updated copyright header. Added reference to COPYRIGHT file.
 *
 * Revision 1.3  2009-10-07 14:23:37  uli
 * Switched to logging mechanism provided by the "new" oflog module.
 *
 * Revision 1.2  2005-12-08 16:59:02  meichel
 * Changed include path schema for all DCMTK header files
 *
 * Revision 1.1  2004/10/20 15:48:54  meichel
 * Initial release of module dcmjp2k, a JPEG 2000 codec for DCMTK.
 *
 *
 */
