/*
 *
 *  Copyright (C) 1997-2010, OFFIS e.V.
 *  All rights reserved.  See COPYRIGHT file for details.
 *
 *  This software and supporting documentation were developed by
 *
 *    OFFIS e.V.
 *    R&D Division Health
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *
 *  Module:  dcmjp2k
 *
 *  Author:  Marco Eichelberg
 *
 *  Purpose: singleton class that registers the decoder for all supported JPEG 2000 processes.
 *
 *  Last Update:      $Author$
 *  Update Date:      $Date$
 *  CVS/RCS Revision: $Revision$
 *  Status:           $State$
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef D2INIT_H
#define D2INIT_H

#include "dcmtk/config/osconfig.h"
#include "dcmtk/ofstd/ofcond.h"  /* for OFCondition */

/** helper class that allows to initialize the underlying JPEG 2000 library
 */
class D2JPEG2000Library
{
public:

  /// default constructor
  D2JPEG2000Library() {}
  
  /// destructor
  ~D2JPEG2000Library() {}
  
  /** get library version number string
   */
  static const char *libraryVersion();

  /** initialize JPEG 2000 library 
   *  @return EC_Normal if successful, an error code otherwise
   */  
  static OFCondition initialize();
  
  /** cleanup JPEG 2000 library
   */
  static void cleanup();
};

#endif

/*
 * CVS/RCS Log
 * $Log$
 * Revision 1.3  2010-10-14 09:28:07  joergr
 * Updated copyright header. Added reference to COPYRIGHT file.
 *
 * Revision 1.2  2005/12/08 16:59:04  meichel
 * Changed include path schema for all DCMTK header files
 *
 * Revision 1.1  2004/10/20 15:48:54  meichel
 * Initial release of module dcmjp2k, a JPEG 2000 codec for DCMTK.
 *
 *
 */
