/*
 *
 *  Copyright (C) 1997-2010, OFFIS e.V.
 *  All rights reserved.  See COPYRIGHT file for details.
 *
 *  This software and supporting documentation were developed by
 *
 *    OFFIS e.V.
 *    R&D Division Health
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *
 *  Module:  dcmjp2k
 *
 *  Author:  Marco Eichelberg
 *
 *  Purpose: JasPer stream handler class for decoding JPEG 2000 images
 *
 *  Last Update:      $Author$
 *  Update Date:      $Date$
 *  CVS/RCS Revision: $Revision$
 *  Status:           $State$
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef D2RDSTRM_H
#define D2RDSTRM_H

#include "dcmtk/config/osconfig.h"
#include "dcmtk/ofstd/oftypes.h"
#include <jasper/jasper.h>

class DcmPixelSequence;

/** stream handler class for decoding JPEG 2000 images.
 *  Implements the stream interface that JasPer expects for reading.
 */
class D2ReadStream
{
public:
  /** constructor
   *  @param pixSeq pixel sequence to read from
   *  @param startItem index of first item to read from
   */
  D2ReadStream(DcmPixelSequence *pixSeq, Uint32 startItem);

  /// destructor
  ~D2ReadStream() {};

  /** read bytes into given buffer
   *  @param buf buffer to read into
   *  @param cnt number of bytes to read
   *  @return number of bytes read
   */
  int read(char *buf, int cnt);

  /** seek within buffer
   *  @param offset seek offset
   *  @param origin type of seek operation, SEEK_SET, SEEK_END or SEEK_CUR
   *  @return new position within buffer, -1 if unreachable
   */
  long seek(long offset, int origin);

  /** return the index of the item from which decoding of the
   *  next frame can start.
   *  @return index of next item
   */
  Uint32 getNextItem() const
  {
    return currentItem_+1;
  }

  /** create JasPer stream object maintained by this class
   *  @param pixSeq DICOM pixel sequence to read from
   *  @param startItem index of first item to read from
   *  @return pointer to JasPer stream structure
   */
  static jas_stream_t *createStream(DcmPixelSequence *pixSeq, Uint32 startItem);

  /** destroy (close and delete) stream object maintained by this class
   *  @param stream pointer to object create by D2ReadStream::createStream()
   *  @return index of first item not used by this buffer
   */
  static Uint32 destroyStream(jas_stream_t *stream);

private:
  /// private unimplemented copy constructor
  D2ReadStream(const D2ReadStream& arg);

  /// private unimplemented copy assignment operator
  D2ReadStream& operator=(const D2ReadStream& arg);

  /// pixel sequence to read from
  DcmPixelSequence *pixSeq_;

  /// index of first item
  Uint32 firstItem_;

  /// offset relative to start of first item
  Uint32 totalOffset_;

  /// index of current item
  Uint32 currentItem_;

  /// offset within current fragment
  Uint32 currentFragmentOffset_;

  /// size of current fragment in bytes
  Uint32 currentFragmentSize_;

  /// pointer to current fragment  
  Uint8 *currentFragment_;
};

#endif

/*
 * CVS/RCS Log
 * $Log$
 * Revision 1.3  2010-10-14 09:28:07  joergr
 * Updated copyright header. Added reference to COPYRIGHT file.
 *
 * Revision 1.2  2005/12/08 16:59:05  meichel
 * Changed include path schema for all DCMTK header files
 *
 * Revision 1.1  2004/10/20 15:48:54  meichel
 * Initial release of module dcmjp2k, a JPEG 2000 codec for DCMTK.
 *
 *
 */
