/*
 *
 *  Copyright (C) 1997-2010, OFFIS e.V.
 *  All rights reserved.  See COPYRIGHT file for details.
 *
 *  This software and supporting documentation were developed by
 *
 *    OFFIS e.V.
 *    R&D Division Health
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *
 *  Module:  dcmjp2k
 *
 *  Author:  Marco Eichelberg
 *
 *  Purpose: JasPer stream handler class for encoding JPEG 2000 images
 *
 *  Last Update:      $Author$
 *  Update Date:      $Date$
 *  CVS/RCS Revision: $Revision$
 *  Status:           $State$
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef D2WRSTRM_H
#define D2WRSTRM_H

#include "dcmtk/config/osconfig.h"
#include "dcmtk/ofstd/oftypes.h"
#include "dcmtk/ofstd/oflist.h"
#include "dcmtk/ofstd/ofcond.h"
#include "dcmtk/dcmdata/dcofsetl.h"

#include <jasper/jasper.h>

class DcmPixelSequence;

/** stream handler class for encoding JPEG 2000 images.
 *  Implements the stream interface that JasPer expects for writing.
 */
class D2WriteStream
{
public:
  /** constructor
   *  @param pixSeq pixel sequence to read from
   *  @param startItem index of first item to read from
   */
  D2WriteStream();

  /// destructor
  ~D2WriteStream();

  /** write given buffer into stream
   *  @param buf buffer to read from
   *  @param cnt number of bytes to copy
   *  @return number of bytes copied
   */
  int write(char *buf, int cnt);

  /** seek within buffer
   *  @param offset seek offset
   *  @param origin type of seek operation, SEEK_SET, SEEK_END or SEEK_CUR
   *  @return new position within buffer, -1 if unreachable
   */
  long seek(long offset, int origin);

  /** store the compressed frame in the given pixel sequence
   *  @param pxSeq pointer to pixel sequence object to write to
   *  @param offsetList list of frame offsets, passed through to pxSeq
   *  @param fragmentSize frame size in kByte, passed through to pxSeq
   *  @return EC_Normal if successful, an error code otherwise
   */
  OFCondition storeCompressedFrame(
      DcmPixelSequence *pxSeq,
      DcmOffsetList& offsetList,
      Uint32 fragmentSize) const;

  /** returns the number of bytes stored in the stream
   *  @return number of bytes stored
   */
  size_t size() const;
  
  /** create JasPer stream object maintained by this class
   *  @param pixSeq DICOM pixel sequence to read from
   *  @param startItem index of first item to read from
   *  @return pointer to JasPer stream structure
   */
  static jas_stream_t *createStream();

  /** destroy (close and delete) stream object maintained by this class
   *  @param stream pointer to object create by D2ReadStream::createStream()
   */
  static void destroyStream(jas_stream_t *stream);

  /** return pointer to stream object maintained by given parameter
   *  @return pointer to stream object
   */
  static const D2WriteStream *getStream(jas_stream_t *stream);

private:

  /// list of compressed pixel data blocks  
  OFList<unsigned char *> pixelDataList_;

  /// filled number of bytes in last block in pixelDataList
  size_t bytesInLastBlock_;

  /// index of current block
  size_t currentBlock_;

  /// offset within current fragment
  size_t currentOffset_;

};

#endif

/*
 * CVS/RCS Log
 * $Log$
 * Revision 1.3  2010-10-14 09:28:07  joergr
 * Updated copyright header. Added reference to COPYRIGHT file.
 *
 * Revision 1.2  2005/12/08 16:59:08  meichel
 * Changed include path schema for all DCMTK header files
 *
 * Revision 1.1  2004/10/20 15:48:54  meichel
 * Initial release of module dcmjp2k, a JPEG 2000 codec for DCMTK.
 *
 *
 */
